<?php
/*
Plugin Name: Custom Post Type Archives
Plugin URI: http://ratvars.com/custom-post-type-archives
Description: Enables archives (with feeds and paging) for custom post types
Author: Rolands Atvars
Version: 1.1.1.1
Author URI: http://ratvars.com
*/
add_action('admin_menu', 'pta_create_menu');

function pta_create_menu() {
	add_submenu_page( // will add submenu under 'Settings' menu
		'options-general.php',
		'Post Type Archives',
		'Post Type Archives',
		'administrator',
		'post-type-archives',
		'pta_render_menu'
	);
	
	add_action('admin_init', 'pta_register_settings');
}

/**
 * Will register all options for this plugin.
 * This way WordPress will deal with all the saving and sanitizing itself.
 * Check 'pta_get_settings' for brief setting explanations.
 * @see pta_get_settings
 */
function pta_register_settings() {
	register_setting('pta-setting-group', 'pta-url-base');
	register_setting('pta-setting-group', 'pta-title');
	register_setting('pta-setting-group', 'pta-template-pattern');
	register_setting('pta-setting-group', 'pta-fallback-template');
	register_setting('pta-setting-group', 'pta-enabled-post-type-archives');
}

/**
 * Will get either all settings for this plugin or just the specified one.
 * Will store settings in static variable so that plugin doesn't have to check the values
 * each time someone requests a random option.
 * Possible options:
 *  - url_base : URL prefix for the custom post type archive.
 *  - title : <title> attribute's value for custom post type archives.
 *    	Will replace {POST_TYPE_NAME} with post types name and {POST_TYPE_SINGULAR_NAME} with post types singular name
 *  - template_pattern : pattern for custom post type archive template. Will replace {POST_TYPE} with current post type
 *  - fallback_template : if no custom post template is found will try to load this file
 *  - enabled_post_type_archives : post types with archives enabled
 * @param string $option optional option name to return
 * @return mixed either all the options (if $option is false or non existing) or specified $option
 */
function pta_get_settings($option = false) {
	static $settings; // static to store settings so that plugin doesn't have to pull and parse them on every call
	
	if(!isset($settings)) { // this, probably, is the first function call in this page load
		$url_base = get_option('pta-url-base', '/post-type');
		$title = get_option('pta-title', '');
		$template_pattern = get_option('pta-template-pattern', '{POST_TYPE}.php');
		$fallback_template = get_option('pta-fallback-template', 'index.php');
		$enabled_post_type_archives = get_option('pta-enabled-post-type-archives', array());

		if(!is_array($enabled_post_type_archives)) // this has to be an array
			$enabled_post_type_archives = array();
		if(empty($template_pattern))
			$template_pattern = '{POST_TYPE}.php';
		if(empty($fallback_template)) // I think that index.php should always be present
			$fallback_template = 'index.php';
		$url_base = trim($url_base, '/'); // trim un-necessary slashes
		$title = htmlspecialchars(trim($title)); // no HTML and whitespaces (in beginning and end)

		$settings = compact(
			'url_base',
			'title',
			'template_pattern',
			'fallback_template',
			'enabled_post_type_archives'
		);
	}
	
	return ($option == false or !isset($settings[$option])) ? $settings : $settings[$option];
}

function pta_render_menu() {
	extract(pta_get_settings());
	
	include dirname(__FILE__) . '/options.php';
}

/**
 * This is a callback function to enable custom post type archive pages.
 * This function will add new rewrite rules to the global $wp_rewrite variable.
 * Custom rewrite rules will support feeds and paging without any extra coding. (yeeey)
 * This function will be called by 'generate_rewrite_rules' filter.
 * Note that it is necessary to flush permalinks for this method to be called.
 * It might be useful to know that these rewrite rules will create new param called 'post_type_index'.
 * It can be used to check whether we are in custom post archive.
 * @param WP_Rewrite $wp_rewrite object that contains all the routing data
 * @return WP_Rewrite modified $wp_rewrite with my custom rewrite rules
 */
function pta_register_post_type_rewrite_rules($wp_rewrite) {
	extract(pta_get_settings());
	if(empty($enabled_post_type_archives)) return $wp_rewrite; // if we don't have any enabled post type archives then might as well quit
	
	$custom_rules = array();
	$enabled_post_type_archives = implode('|', $enabled_post_type_archives);
	$url_base = ($url_base == '') ? $url_base : $url_base . '/';
	
	$custom_rules = array(
		"$url_base($enabled_post_type_archives)/page/?([0-9]{1,})/?$" => // enable paging
			'index.php?post_type_index=1&post_type=' . $wp_rewrite->preg_index(1) . '&paged=' . $wp_rewrite->preg_index(2),
		"$url_base($enabled_post_type_archives)/(feed|rdf|rss|rss2|atom)/?$" => // enable feeds
			'index.php?post_type_index=1&post_type=' . $wp_rewrite->preg_index(1) . '&feed=' . $wp_rewrite->preg_index(2),
		"$url_base($enabled_post_type_archives)/?$" => // enable the index page
			'index.php?post_type_index=1&post_type=' . $wp_rewrite->preg_index(1)
	);
	
	$wp_rewrite->rules = array_merge($custom_rules, $wp_rewrite->rules); // merge existing rules with custom ones
	
	return $wp_rewrite;
}
add_filter('generate_rewrite_rules', 'pta_register_post_type_rewrite_rules');

/**
 * This hook callback function will try to tell WordPress to include proper template
 * when in a custom post type archive. (created by 'pta_register_post_type_rewrite_rules')
 * Function will use locate_template to check whether the template exists.
 * Hook that calls this function is 'template_include'
 * @see pta_register_post_type_rewrite_rules
 * @uses locate_template
 * @link http://codex.wordpress.org/Function_Reference/locate_template
 * @param string $template path to template that WordPress thinks is necessary to include
 * @return string path to template to include. Will be changed if we're in custom post type area.
 */
function pta_register_post_type_redirects($template) {
	if(!is_post_type_archive()) return $template;
	
	extract(pta_get_settings()); // let's get the settings first
	
	$current_post_type = get_query_var('post_type'); // and get current custom post
	$template_pattern = str_replace('{POST_TYPE}', $current_post_type, $template_pattern); // create our template pattern
	$template_path = locate_template((array) $template_pattern); // and try to find the template file
	$fallback_template_path = locate_template((array) $fallback_template); // for safety lets find fallback template
	
	if($template_path != '')
		return $template_path;
	elseif($fallback_template_path != '')
		return $fallback_template_path;
	
	return $template; // if everything else fails then continue with WordPress idea
}
add_filter('template_include', 'pta_register_post_type_redirects');

/**
 * For some reason WordPress thinks that custom post archives are is_home()
 * They are not. Must fix this!
 */
function pta_fix_post_type_context() {
	if(is_post_type_archive()) {
		global $wp_query;
		
		$wp_query->is_home = false;
	}
}
add_filter('template_redirect', 'pta_fix_post_type_context');

/**
 * By default WordPress will have no title for custom post type archives.
 * This is where this fix comes in! ^_^
 * @uses option 'Title'
 * Will only change the title if we are in post type archive.
 * Won't change the title if saved 'Title' options value is empty.
 * Won't change the title if we are on non existing post type (don't think that this can happen though :P)
 * Will replace {POST_TYPE_NAME} with custom post types name.
 * Will replace {POST_TYPE_SINGULAR_NAME} with custom post types singular name.
 * This filter will tolorate the $seplocation.
 * @param string $wp_title prepared WordPress title (will be empty unless some other plugin wants to be smart)
 * @param string $sep the defined seperator
 * @param string $seplocation where is the seperator placed.
 */
function pta_fix_wp_title($wp_title, $sep, $seplocation) {
	if(is_post_type_archive()) { // do stuff if we are in post type archive
		$title = pta_get_settings('title'); // let's get the title now
		if(empty($title)) return $wp_title; // ups - it's empty. Don't continue!
		
		$current_post_type = get_post_types(array(), 'object'); // lets get all the post types
		if(!isset($current_post_type[get_query_var('post_type')])) return $wp_title; // ups current one isn't in that list. END NOW!
		
		$current_post_type = $current_post_type[get_query_var('post_type')]; // ok it exists. Select it!
		
		$title = str_replace(
			'{POST_TYPE_NAME}',
			$current_post_type->labels->name,
			$title
		);
		$title = str_replace(
			'{POST_TYPE_SINGULAR_NAME}',
			$current_post_type->labels->singular_name,
			$title
		);
		
		if($seplocation == 'right') // this is the right case - OK - just append
			$wp_title = "$title $sep ";
		else // otherwise preppend. I used WordPress doc so this is all valid :)
			$wp_title = " $sep $title";
	}
	
	return $wp_title; // we aren't in post type archive - quit FAST!
}
add_filter('wp_title', 'pta_fix_wp_title', 10, 3);

/**
 * Similar function as 'is_home' or 'is_category'.
 * Will tell you if this is custom post type archive page.
 * Uses the custom rewrite rule created 'post_type_index' param.
 * If that param exists and equals to '1' then this is a custom post type archive.
 * Will also check if this custom post is saved as a enabled post type archive
 * Will store the result in static variable so that plugin doesn't have to do this calculation on every call.
 * @global $wp
 * @return bool
 */
function is_post_type_archive() {
	static $is_post_type_index;
	
	if(!isset($is_post_type_index)) {
		global $wp; // this is where the matched query is stored

		$q = $wp->matched_query;
		parse_str($q, $q); // make query string into an array

		$post_type_index = isset($q['post_type_index']) ? $q['post_type_index'] : false;
		$post_type = get_query_var('post_type');
		$enabled_post_type_archives = pta_get_settings('enabled_post_type_archives');
	
		if($post_type_index == '1' and in_array($post_type, $enabled_post_type_archives))
			$is_post_type_index = true;
		else
			$is_post_type_index = false;
	}
	
	return $is_post_type_index;
}

/**
 * Just a wrapper around get_the_post_type_permalink to quickly echo the link.
 * @uses get_the_post_type_permalink
 */
function the_post_type_permalink($post_type = false) {
	echo get_the_post_type_permalink($post_type);
}

/**
 * Will return a link to custom post type archive.
 * If used in a loop then there is no need to specify $post_type
 * @param string|int|object|bool $post_type optional post type to link to.
 * 	If nothing or false is passed then assumes, that we are in a loop.
 * 	If int or object is passed then assumes that that's an post ID or post object and gets post type from that.
 * 	If string is passed then assumes that that's post types slug and uses that.
 * @return string will return link to post type or empty string if no post type was found.
 */
function get_the_post_type_permalink($post_type = false) {
	if($post_type === false) // if there is no post type passed then assume that we are in the WP loop
		$post_type = get_post_type();
	elseif(is_int($post_type) or is_object($post_type)) // if we have something and its either an ID or post object then get it's post type
		$post_type = get_post_type($post_type);
		
	if(empty($post_type)) // if we didn't get a post type (and it wasn't passed), then just return an empty string
		return '';
	
	$url_base = pta_get_settings('url_base');
	if($url_base == '')
		$url_base = '/';
	else
		$url_base = "/$url_base/";
	
	$url = get_bloginfo('url') . $url_base . $post_type . '/';
	
	return $url;
}
?>